import { __decorate, __param } from 'tslib';
import { InjectionToken, EventEmitter, Inject, Input, Output, Component, NgModule } from '@angular/core';
import { Validators, FormBuilder, ReactiveFormsModule, FormsModule } from '@angular/forms';
import { validateCardNumber, parseCardType } from 'payform';
import { IonicModule } from '@ionic/angular';
import { CommonModule } from '@angular/common';

var ION_CUSTOM_FORM_BUILDER_CONFIG = new InjectionToken('ION_CUSTOM_FORM_BUILDER_CONFIG');

var IonCustomFormBuilderComponent = /** @class */ (function () {
    function IonCustomFormBuilderComponent(formBuilder, config) {
        this.formBuilder = formBuilder;
        this.config = config;
        this.formControls = {};
        this.formBuilt = false;
        this.emailRegEx = '^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}$';
        this.creditCardImg = 'generic.svg';
        this.formFields = [];
        this.submitButtonText = 'Submit';
        this.errorsIndex = [];
        this.defaultCssClass = this.config ? this.config.defaultCssClass : undefined;
        this.successCssClass = this.config ? this.config.successCssClass : undefined;
        this.errorCssClass = this.config ? this.config.errorCssClass : undefined;
        this.showLabels = true;
        this.showIcons = true;
        this.showCardIcons = true;
        this.returnCreditCardType = false;
        /* supports ionic theme colors defined in theme/variables.css i.e 'primary', 'secondary' */
        this.iconColor = this.config ? this.config.defaultIconColor : undefined;
        this.submitButtonColor = this.config ? this.config.submitButtonColor : undefined;
        this.formSubmission = new EventEmitter();
        this.masks = {
            cardExpiry: [/[0-1]/, /\d/, '/', /[1-9]/, /\d/],
            cardNumber: [/\d/, /\d/, /\d/, /\d/, ' ', /\d/, /\d/, /\d/, /\d/, ' ', /\d/, /\d/, /\d/, /\d/, ' ', /\d/, /\d/, /\d/, /\d/],
            dateAutoCorrect: [/\d/, /\d/, '/', /\d/, /\d/, '/', /\d/, /\d/, /\d/, /\d/]
        };
    }
    IonCustomFormBuilderComponent.prototype.ngOnChanges = function () {
        var _this = this;
        this.errorsIndex.forEach(function (index) {
            _this.formFields[index].errors = true;
        });
    };
    IonCustomFormBuilderComponent.prototype.ngOnInit = function () {
        var _this = this;
        // tslint:disable-next-line: no-shadowed-variable
        this.formFields.forEach(function (element, index, arr) {
            _this.formControls["" + element.formControlName] = ['', element.validators];
            if (element.type === 'email') {
                _this.formFields[index].validators.push(Validators.pattern(_this.emailRegEx));
            }
        });
        this.customForm = this.formBuilder.group(this.formControls);
        // tslint:disable-next-line: no-shadowed-variable
        this.formFields.forEach(function (element, index, arr) {
            _this.formFields[index].control = _this.customForm.controls["" + element.formControlName];
            if (element.placeholder !== undefined) {
                _this.formFields[index].control.setValue(element.formFieldValue);
                _this.formFields[index].control.markAsTouched();
            }
            if (element.formFieldType === 'card') {
                _this.creditCardFieldIndex = index;
            }
        });
        this.formBuilt = true;
        this.customForm.valueChanges.subscribe(function (result) {
            Object.keys(result).forEach(function (key, index, arr) {
                if (result.hasOwnProperty('password') && result.hasOwnProperty('confirm_password')) {
                    var passwordIndex = arr.indexOf('password');
                    var confirmPasswordIndex = arr.indexOf('confirm_password');
                    _this.handlePasswords(passwordIndex, confirmPasswordIndex);
                }
            });
        });
        if (this.creditCardFieldIndex !== undefined) {
            this.formFields[this.creditCardFieldIndex].control.valueChanges.subscribe(function (cardNumber) {
                _this.validateCardNumber(cardNumber);
                _this.detectCardType(cardNumber);
            });
        }
    };
    IonCustomFormBuilderComponent.prototype.validateCardNumber = function (cardNumber) {
        if (validateCardNumber(cardNumber) === false) {
            this.formFields[this.creditCardFieldIndex].errors = true;
        }
        else {
            this.formFields[this.creditCardFieldIndex].errors = false;
        }
    };
    IonCustomFormBuilderComponent.prototype.detectCardType = function (cardNumber) {
        if (parseCardType(cardNumber) === 'visa') {
            this.creditCardImg = 'visa.svg';
        }
        else if (parseCardType(cardNumber) === 'amex') {
            this.creditCardImg = 'amex.svg';
        }
        else if (parseCardType(cardNumber) === 'mastercard') {
            this.creditCardImg = 'master.svg';
        }
        else {
            this.creditCardImg = 'generic.svg';
        }
    };
    IonCustomFormBuilderComponent.prototype.createCardIcon = function (icon) {
        var iconUrl = "assets/" + icon;
        return {
            // tslint:disable-next-line: object-literal-key-quotes
            'width': '14%',
            'margin-top': icon === 'master.svg' ? '10px' : '5px',
            'margin-right': '12px',
            // tslint:disable-next-line: object-literal-key-quotes
            'content': "url(" + iconUrl + ")"
        };
    };
    IonCustomFormBuilderComponent.prototype.handlePasswords = function (passwordIndex, confirmPasswordIndex) {
        if (this.formFields[passwordIndex].control.value.length > 0 &&
            this.formFields[passwordIndex].control.value !== this.formFields[confirmPasswordIndex].control.value) {
            this.formFields[passwordIndex].title = this.formFields[confirmPasswordIndex].title = 'Passwords don\'t match';
            this.formFields[passwordIndex].errors = this.formFields[confirmPasswordIndex].errors = true;
        }
        else {
            this.formFields[passwordIndex].title = 'Password';
            this.formFields[confirmPasswordIndex].title = 'Confirm Password';
            this.formFields[passwordIndex].errors = this.formFields[confirmPasswordIndex].errors = false;
        }
    };
    IonCustomFormBuilderComponent.prototype.submitForm = function () {
        var _this = this;
        var formData = {};
        // tslint:disable-next-line: no-shadowed-variable
        this.formFields.forEach(function (element, index, arr) {
            formData["" + element.formControlName] = _this.formFields[index].control.value;
        });
        if (formData.hasOwnProperty('confirm_password')) {
            // tslint:disable-next-line: no-string-literal
            delete formData['confirm_password'];
        }
        var detectCardFormFieldArr = [];
        detectCardFormFieldArr = this.formFields.filter(function (element, index, arr) {
            return element.formFieldType === 'card';
        });
        if (detectCardFormFieldArr.length > 0 && this.returnCreditCardType === true) {
            // tslint:disable-next-line: no-string-literal
            formData['card_type'] = parseCardType(formData['card']);
        }
        this.formSubmission.emit(formData);
    };
    IonCustomFormBuilderComponent.prototype.hasValidationErrors = function () {
        var _this = this;
        var controlErrors = [];
        Object.keys(this.customForm.controls).forEach(function (key) {
            controlErrors.push(_this.customForm.get(key).errors);
        });
        // tslint:disable-next-line: no-shadowed-variable
        var result = controlErrors.filter(function (element, index, arr) {
            return element !== null;
        });
        return result.length !== 0;
    };
    IonCustomFormBuilderComponent.prototype.hasFieldErrors = function () {
        // tslint:disable-next-line: no-shadowed-variable
        var fieldErrors = this.formFields.filter(function (element, index, arr) {
            return element.errors === true;
        });
        return fieldErrors.length !== 0;
    };
    IonCustomFormBuilderComponent.prototype.setCssClasses = function (field) {
        var classes = {};
        // tslint:disable-next-line: no-string-literal
        classes[this.defaultCssClass ? "" + this.defaultCssClass : 'default-form-input'] = true;
        // tslint:disable-next-line: max-line-length
        classes[this.errorCssClass ? "" + this.errorCssClass : 'default-form-input-error'] = field.control.touched && field.control.errors || field.errors ? true : false;
        // tslint:disable-next-line: max-line-length
        classes[this.successCssClass ? "" + this.successCssClass : 'default-form-input-success'] = !field.control.errors && !field.errors ? true : false;
        return classes;
    };
    IonCustomFormBuilderComponent.ctorParameters = function () { return [
        { type: FormBuilder },
        { type: undefined, decorators: [{ type: Inject, args: [ION_CUSTOM_FORM_BUILDER_CONFIG,] }] }
    ]; };
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "formFields", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "submitButtonText", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "errorsIndex", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "defaultCssClass", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "successCssClass", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "errorCssClass", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "showLabels", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "showIcons", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "showCardIcons", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "returnCreditCardType", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "iconColor", void 0);
    __decorate([
        Input()
    ], IonCustomFormBuilderComponent.prototype, "submitButtonColor", void 0);
    __decorate([
        Output()
    ], IonCustomFormBuilderComponent.prototype, "formSubmission", void 0);
    IonCustomFormBuilderComponent = __decorate([
        Component({
            selector: 'ion-custom-form-builder',
            template: "<form\n  [formGroup]=\"customForm\"\n  novalidate\n  *ngIf=\"formBuilt\"\n  (ngSubmit)=\"submitForm()\"\n>\n  <ion-row\n    justify-content-center\n    align-items-center\n    *ngFor=\"let field of formFields\"\n  >\n    <!-- Inline Text field controller -->\n    <ion-item\n      *ngIf=\"\n        field.formFieldType === 'inline' || field.formFieldType === undefined\n      \"\n      lines=\"none\"\n      [ngClass]=\"setCssClasses(field)\"\n    >\n      <ion-icon slot=\"start\" name=\"{{ field.icon }}\" color=\"{{iconColor ? iconColor : 'primary'}}\" *ngIf=\"showIcons\"></ion-icon>\n      <ion-label position=\"{{field?.labelPosition ? field?.labelPosition : 'floating'}}\" *ngIf=\"showLabels\">{{ field.title }}</ion-label>\n      <ion-input\n        *ngIf=\"field?.placeholder\"\n        mode=\"md\"\n        placeholder=\"{{field?.placeholder}}\"\n        type=\"{{ field.type }}\"\n        formControlName=\"{{ field.formControlName }}\"\n      ></ion-input>\n      <ion-input\n        *ngIf=\"field?.placeholder === undefined\"\n        mode=\"md\"\n        type=\"{{ field.type }}\"\n        formControlName=\"{{ field.formControlName }}\"\n      ></ion-input>\n    </ion-item>\n\n    <!-- Credit card field controller -->\n    <ion-item\n      *ngIf=\"\n        field.formFieldType === 'card'\n      \"\n      lines=\"none\"\n      [ngClass]=\"setCssClasses(field)\"\n    >\n      <div\n        *ngIf=\"showCardIcons\"\n        slot=\"start\"\n        [ngStyle]=\"createCardIcon(creditCardImg)\"\n      ></div>\n      <ion-label position=\"{{field?.labelPosition ? field?.labelPosition : 'floating'}}\" *ngIf=\"showLabels\">{{ field.title }}</ion-label>\n      <ion-input\n        *ngIf=\"field?.placeholder\"\n        mode=\"md\"\n        placeholder=\"{{field?.placeholder}}\"\n        type=\"{{ field.type }}\"\n        formControlName=\"{{ field.formControlName }}\"\n      ></ion-input>\n      <ion-input\n        *ngIf=\"field?.placeholder === undefined\"\n        mode=\"md\"\n        type=\"{{ field.type }}\"\n        formControlName=\"{{ field.formControlName }}\"\n      ></ion-input>\n    </ion-item>\n\n    <!-- Text Area Controller -->\n    <ion-item\n      *ngIf=\"field.formFieldType === 'textarea'\"\n      lines=\"none\"\n      [ngClass]=\"setCssClasses(field)\"\n    >\n      <ion-label position=\"{{field?.labelPosition ? field?.labelPosition : 'floating'}}\" *ngIf=\"showLabels\">{{ field.title }}</ion-label>\n      <ion-textarea\n        *ngIf=\"field?.placeholder\"\n        mode=\"md\"\n        rows=\"{{ field?.textAreaRowCount }}\"\n        formControlName=\"{{ field.formControlName }}\"\n        placeholder=\"{{field?.placeholder}}\"\n      ></ion-textarea>\n      <ion-textarea\n        *ngIf=\"field?.placeholder === undefined\"\n        mode=\"md\"\n        rows=\"{{ field?.textAreaRowCount }}\"\n        formControlName=\"{{ field.formControlName }}\"\n      ></ion-textarea>\n    </ion-item>\n  </ion-row>\n\n  <!-- Submit Button -->\n  <ion-row align-items-center justify-content-center margin-top>\n    <ion-button\n      color=\"{{submitButtonColor ? submitButtonColor : 'primary'}}\"\n      type=\"submit\"\n      [disabled]=\"hasValidationErrors() || hasFieldErrors()\"\n    >\n      {{ submitButtonText }}\n    </ion-button>\n  </ion-row>\n</form>\n",
            styles: [".default-form-input{border:1px solid #007bff;border-radius:.5rem;margin-top:16px;width:80%}.default-form-input-error{border:1px solid #dc3545}.default-form-input-success{border:1px solid #28a745}"]
        }),
        __param(1, Inject(ION_CUSTOM_FORM_BUILDER_CONFIG))
    ], IonCustomFormBuilderComponent);
    return IonCustomFormBuilderComponent;
}());

var Components = [
    IonCustomFormBuilderComponent
];
var IonCustomFormBuilderModule = /** @class */ (function () {
    function IonCustomFormBuilderModule() {
    }
    IonCustomFormBuilderModule_1 = IonCustomFormBuilderModule;
    IonCustomFormBuilderModule.forRoot = function (options) {
        return {
            ngModule: IonCustomFormBuilderModule_1,
            providers: [
                {
                    provide: ION_CUSTOM_FORM_BUILDER_CONFIG,
                    useValue: options
                }
            ]
        };
    };
    var IonCustomFormBuilderModule_1;
    IonCustomFormBuilderModule = IonCustomFormBuilderModule_1 = __decorate([
        NgModule({
            declarations: [
                Components
            ],
            imports: [
                IonicModule,
                CommonModule,
                ReactiveFormsModule,
                FormsModule,
            ],
            exports: [
                Components
            ]
        })
    ], IonCustomFormBuilderModule);
    return IonCustomFormBuilderModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { IonCustomFormBuilderModule, IonCustomFormBuilderComponent as ɵa, ION_CUSTOM_FORM_BUILDER_CONFIG as ɵb };
//# sourceMappingURL=ion-custom-form-builder.js.map
